@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2007 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2007-2008 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -23,6 +23,7 @@
 
 #include <errno.h>
 #include <string.h>
+#include <signal.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <glib.h>
@@ -30,31 +31,36 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
-#include <pk-debug.h>
+#include "egg-debug.h"
+#include "egg-dbus-monitor.h"
+
 #include "pk-conf.h"
 #include "pk-engine.h"
+#include "pk-transaction.h"
+#include "pk-backend-internal.h"
 #include "pk-interface.h"
 
 static guint exit_idle_time;
+static PkEngine *engine = NULL;
+static PkBackend *backend = NULL;
 
 /**
  * pk_object_register:
  * @connection: What we want to register to
  * @object: The GObject we want to register
  *
- * Register org.freedesktop.PowerManagement on the session bus.
+ * Register org.freedesktop.PackageKit on the system bus.
  * This function MUST be called before DBUS service will work.
  *
  * Return value: success
  **/
-static gboolean
-pk_object_register (DBusGConnection *connection,
-		    GObject	     *object,
-		    GError **error)
+G_GNUC_WARN_UNUSED_RESULT static gboolean
+pk_object_register (DBusGConnection *connection, GObject *object, GError **error)
 {
 	DBusGProxy *bus_proxy = NULL;
 	guint request_name_result;
 	gboolean ret;
+	gchar *message;
 
 	bus_proxy = dbus_g_proxy_new_for_name (connection,
 					       DBUS_SERVICE_DBUS,
@@ -67,33 +73,35 @@ pk_object_register (DBusGConnection *connection,
 				 G_TYPE_INVALID,
 				 G_TYPE_UINT, &request_name_result,
 				 G_TYPE_INVALID);
-	if (error && *error) {
-		pk_debug ("ERROR: %s", (*error)->message);
-	}
-	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
-		pk_warning ("RequestName failed!");
-		g_clear_error(error);
-		g_set_error(error, PK_ENGINE_ERROR, PK_ENGINE_ERROR_DENIED,
-			    "Acquiring D-Bus name %s failed due to security policies on this machine\n"
-			    "This can happen for two reasons:\n"
-			    "* The correct user is not launching the executable (usually root)\n"
-			    "* The org.freedesktop.PackageKit.conf file is "
-			    "not installed in the system /etc/dbus-1/system.d directory\n", PK_DBUS_SERVICE);
-		return FALSE;
-	}
 
 	/* free the bus_proxy */
 	g_object_unref (G_OBJECT (bus_proxy));
 
+	/* abort as the DBUS method failed */
+	if (!ret) {
+		egg_warning ("RequestName failed!");
+		g_clear_error (error);
+		message = g_strdup_printf ("%s\n%s\n* %s\n* %s\n",
+					   _("Startup failed due to security policies on this machine."),
+					   _("This can happen for two reasons:"),
+					   _("The correct user is not launching the executable (usually root)"),
+					   _("The org.freedesktop.PackageKit.conf file is not "
+					     "installed in the system /etc/dbus-1/system.d directory"));
+		g_set_error (error, PK_ENGINE_ERROR, PK_ENGINE_ERROR_DENIED, "%s", message);
+		g_free (message);
+		return FALSE;
+	}
+
 	/* already running */
  	if (request_name_result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
-		g_set_error(error, PK_ENGINE_ERROR, PK_ENGINE_ERROR_DENIED, "Already running on this machine");
+		g_set_error (error, PK_ENGINE_ERROR, PK_ENGINE_ERROR_DENIED,
+			     "Already running on this machine");
 		return FALSE;
 	}
 
 	dbus_g_object_type_install_info (PK_TYPE_ENGINE, &dbus_glib_pk_engine_object_info);
 	dbus_g_error_domain_register (PK_ENGINE_ERROR, NULL, PK_ENGINE_TYPE_ERROR);
+	dbus_g_error_domain_register (PK_TRANSACTION_ERROR, NULL, PK_TRANSACTION_TYPE_ERROR);
 	dbus_g_connection_register_g_object (connection, PK_DBUS_PATH, object);
 
 	return TRUE;
@@ -122,14 +130,37 @@ pk_main_timeout_check_cb (PkEngine *engine)
 {
 	guint idle;
 	idle = pk_engine_get_seconds_idle (engine);
-	pk_debug ("idle is %i", idle);
+	egg_debug ("idle is %i", idle);
 	if (idle > exit_idle_time) {
-		pk_warning ("exit!!");
+		egg_warning ("exit!!");
 		exit (0);
 	}
 	return TRUE;
 }
 
+/**
+ * pk_main_sigint_handler:
+ **/
+static void
+pk_main_sigint_handler (int sig)
+{
+	egg_debug ("Handling SIGINT");
+
+	/* restore default ASAP, as the finalisers might hang */
+	signal (SIGINT, SIG_DFL);
+
+	/* cleanup */
+	g_object_unref (backend);
+	g_object_unref (engine);
+
+	/* give the backend a sporting chance */
+	g_usleep (500*1000);
+
+	/* kill ourselves */
+	egg_debug ("Retrying SIGINT");
+	kill (getpid (), SIGINT);
+}
+
 /**
  * main:
  **/
@@ -138,33 +169,38 @@ main (int argc, char *argv[])
 {
 	GMainLoop *loop;
 	DBusGConnection *system_connection;
+	EggDbusMonitor *monitor;
+	gboolean ret;
 	gboolean verbose = FALSE;
 	gboolean disable_timer = FALSE;
 	gboolean version = FALSE;
 	gboolean use_daemon = FALSE;
 	gboolean timed_exit = FALSE;
 	gboolean immediate_exit = FALSE;
-	gchar *backend = NULL;
+	gboolean do_logging = FALSE;
+	gchar *backend_name = NULL;
 	PkConf *conf = NULL;
-	PkEngine *engine = NULL;
 	GError *error = NULL;
 	GOptionContext *context;
+	const gchar *env_pk_verbose;
+	const gchar *env_pk_console;
+	const gchar *env_pk_logging;
 
 	const GOptionEntry options[] = {
-		{ "backend", '\0', 0, G_OPTION_ARG_STRING, &backend,
-		  "Backend to use (for debugging)", NULL },
+		{ "backend", '\0', 0, G_OPTION_ARG_STRING, &backend_name,
+		  _("Packaging backend to use, e.g. dummy"), NULL },
 		{ "daemonize", '\0', 0, G_OPTION_ARG_NONE, &use_daemon,
-		  "Daemonize and detach", NULL },
+		  _("Daemonize and detach from the terminal"), NULL },
 		{ "verbose", '\0', 0, G_OPTION_ARG_NONE, &verbose,
-		  "Show extra debugging information", NULL },
+		  _("Show extra debugging information"), NULL },
 		{ "disable-timer", '\0', 0, G_OPTION_ARG_NONE, &disable_timer,
-		  "Disable the idle timer", NULL },
+		  _("Disable the idle timer"), NULL },
 		{ "version", '\0', 0, G_OPTION_ARG_NONE, &version,
-		  "Show version of installed program and exit", NULL },
+		  _("Show version and exit"), NULL },
 		{ "timed-exit", '\0', 0, G_OPTION_ARG_NONE, &timed_exit,
-		  "Exit after a small delay (for debugging)", NULL },
+		  _("Exit after a small delay"), NULL },
 		{ "immediate-exit", '\0', 0, G_OPTION_ARG_NONE, &immediate_exit,
-		  "Exit after a the engine has loaded (for debugging)", NULL },
+		  _("Exit after the engine has loaded"), NULL },
 		{ NULL}
 	};
 
@@ -174,88 +210,116 @@ main (int argc, char *argv[])
 	dbus_g_thread_init ();
 	g_type_init ();
 
-	context = g_option_context_new (_("PackageKit daemon"));
+	context = g_option_context_new (_("PackageKit service"));
 	g_option_context_add_main_entries (context, options, NULL);
 	g_option_context_parse (context, &argc, &argv, NULL);
 	g_option_context_free (context);
-	pk_debug_init (verbose);
+	egg_debug_init (verbose);
 
-	if (version == TRUE) {
+	if (version) {
 		g_print ("Version %s\n", VERSION);
-		goto unref_program;
+		goto exit_program;
 	}
 
-	/* force to crash for development releases */
-	setenv ("G_DEBUG", "fatal_criticals", 1);
+	/* check if an instance is already running */
+	monitor = egg_dbus_monitor_new ();
+	egg_dbus_monitor_assign (monitor, EGG_DBUS_MONITOR_SYSTEM, PK_DBUS_SERVICE);
+	ret = egg_dbus_monitor_is_connected (monitor);
+	g_object_unref (monitor);
+	if (ret) {
+		g_print ("Already running service which provides %s\n", PK_DBUS_SERVICE);
+		goto exit_program;
+	}
 
-	if (!g_thread_supported ())
-		g_thread_init (NULL);
-	dbus_g_thread_init ();
+	/* do stuff on ctrl-c */
+	signal (SIGINT, pk_main_sigint_handler);
 
 	/* we need to daemonize before we get a system connection */
-	if (use_daemon == TRUE && daemon (0, 0)) {
-		g_error ("Could not daemonize: %s", g_strerror (errno));
+	if (use_daemon && daemon (0, 0)) {
+		g_print ("Could not daemonize: %s\n", g_strerror (errno));
+		goto exit_program;
 	}
 
+	/* don't let GIO start it's own session bus: http://bugzilla.gnome.org/show_bug.cgi?id=526454 */
+	setenv ("GIO_USE_VFS", "local", 1);
+
 	/* check dbus connections, exit if not valid */
 	system_connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
 	if (error) {
-		pk_warning ("%s", error->message);
+		g_print ("%s: %s\n", _("Cannot connect to the system bus"), error->message);
 		g_error_free (error);
-		g_error ("This program cannot start until you start "
-			   "the dbus system service.\n"
-			   "It is <b>strongly recommended</b> you reboot "
-			   "your computer after starting this service.");
+		goto exit_program;
 	}
 
 	/* we don't actually need to do this, except it rules out the
 	 * 'it works from the command line but not service activation' bugs */
+	env_pk_verbose = g_getenv (EGG_VERBOSE);
+	env_pk_console = g_getenv (EGG_CONSOLE);
+	env_pk_logging = g_getenv (EGG_LOGGING);
 	clearenv ();
+	g_setenv (EGG_VERBOSE, env_pk_verbose, FALSE);
+	g_setenv (EGG_CONSOLE, env_pk_console, FALSE);
+	g_setenv (EGG_LOGGING, env_pk_logging, FALSE);
 
 	/* get values from the config file */
 	conf = pk_conf_new ();
-	exit_idle_time = pk_conf_get_int (conf, "ShutdownTimeout");
-	pk_debug ("daemon shutdown set to %i seconds", exit_idle_time);
 
-	if (backend == NULL) {
-		backend = pk_conf_get_string (conf, "DefaultBackend");
-		pk_debug ("using default backend %s", backend);
+	/* do we log? */
+	do_logging = pk_conf_get_bool (conf, "TransactionLogging");
+	egg_debug ("Log all transactions: %i", do_logging);
+	egg_debug_set_logging (do_logging);
+
+	/* after how long do we timeout? */
+	exit_idle_time = pk_conf_get_int (conf, "ShutdownTimeout");
+	egg_debug ("daemon shutdown set to %i seconds", exit_idle_time);
+
+	if (backend_name == NULL) {
+		backend_name = pk_conf_get_string (conf, "DefaultBackend");
+		egg_debug ("using default backend %s", backend_name);
+	}
+
+	/* load our chosen backend */
+	backend = pk_backend_new ();
+	ret = pk_backend_set_name (backend, backend_name);
+	g_free (backend_name);
+
+	/* all okay? */
+	if (!ret) {
+		egg_error ("cannot continue, backend invalid");
 	}
 
 	/* create a new engine object */
 	engine = pk_engine_new ();
-	pk_engine_use_backend (engine, backend);
 
 	if (!pk_object_register (system_connection, G_OBJECT (engine), &error)) {
-		g_error ("Error trying to start: %s", error->message);
+		g_print (_("Error trying to start: %s\n"), error->message);
 		g_error_free (error);
-		return 0;
+		goto out;
 	}
 
 	loop = g_main_loop_new (NULL, FALSE);
 
 	/* Only timeout and close the mainloop if we have specified it
 	 * on the command line */
-	if (timed_exit == TRUE) {
+	if (timed_exit)
 		g_timeout_add_seconds (20, (GSourceFunc) timed_exit_cb, loop);
-	}
 
 	/* only poll every 10 seconds when we are alive */
-	if (exit_idle_time != 0 && disable_timer == FALSE) {
+	if (exit_idle_time != 0 && !disable_timer)
 		g_timeout_add_seconds (5, (GSourceFunc) pk_main_timeout_check_cb, engine);
-	}
 
 	/* immediatly exit */
-	if (immediate_exit == TRUE) {
+	if (immediate_exit)
 		g_timeout_add (50, (GSourceFunc) timed_exit_cb, loop);
-	}
 
 	g_main_loop_run (loop);
 	g_main_loop_unref (loop);
+
+out:
 	g_object_unref (conf);
 	g_object_unref (engine);
-	g_free (backend);
+	g_object_unref (backend);
 
-unref_program:
+exit_program:
 	return 0;
 }
